﻿using System;
using System.Windows.Forms;
using HeuristicLab.Collections;
using HeuristicLab.Common.Resources;
using HeuristicLab.Core;
using HeuristicLab.Core.Views;
using HeuristicLab.MainForm;
using HeuristicLab.Operators;

namespace HeuristicLab.DebugEngine {

  [View("Operator Trace View")]
  [Content(typeof(ItemList<IOperator>), IsDefaultView = true)]
  public sealed partial class OperatorTraceView : ItemView {

    public new ItemList<IOperator> Content {
      get { return (ItemList<IOperator>)base.Content; }
      set { base.Content = value; }
    }

    public OperatorTraceView() {
      InitializeComponent();
    }

    protected override void DeregisterContentEvents() {
      Content.CollectionReset -= Content_CollectionChanged;
      Content.ItemsAdded -= Content_CollectionChanged;
      Content.ItemsMoved -= Content_CollectionChanged;
      Content.ItemsRemoved -= Content_CollectionChanged;
      Content.ItemsReplaced -= Content_CollectionChanged;
      base.DeregisterContentEvents();
    }

    protected override void RegisterContentEvents() {
      base.RegisterContentEvents();
      Content.CollectionReset += Content_CollectionChanged;
      Content.ItemsAdded += Content_CollectionChanged;
      Content.ItemsMoved += Content_CollectionChanged;
      Content.ItemsRemoved += Content_CollectionChanged;
      Content.ItemsReplaced += Content_CollectionChanged;
    }

    #region Event Handlers (Content)
    void Content_CollectionChanged(object sender, CollectionItemsChangedEventArgs<Collections.IndexedItem<IOperator>> e) {
      if (InvokeRequired)
        Invoke(new EventHandler<CollectionItemsChangedEventArgs<IndexedItem<IOperator>>>(Content_CollectionChanged), sender, e);
      else
        UpdateOperatorTrace();
    }
    #endregion

    private void UpdateOperatorTrace() {
      if (suspended)
        return;
      listView.BeginUpdate();
      listView.Items.Clear();
      listView.SmallImageList.Images.Clear();
      listView.SmallImageList.Images.Add(VS2008ImageLibrary.Method);
      listView.SmallImageList.Images.Add(VS2008ImageLibrary.Module);
      foreach (var item in Content) {
        var viewItem = listView.Items.Add(item.Name ?? item.ItemName);
        viewItem.ToolTipText = string.Format("{0}{1}{1}{2}",
          Utils.TypeName(item), Environment.NewLine,
          Utils.Wrap(item.Description, 60));
        viewItem.Tag = item;
        viewItem.ImageIndex = item is CombinedOperator ? 1 : 0;
      }
      if (listView.Items.Count > 0) {
        for (int i = 0; i < listView.Columns.Count; i++)
          listView.Columns[i].AutoResize(ColumnHeaderAutoResizeStyle.ColumnContent);
      }
      listView.EndUpdate();
    }

    protected override void OnContentChanged() {
      base.OnContentChanged();
      if (Content == null) {
        listView.Items.Clear();
      } else {
        UpdateOperatorTrace();
      }
    }

    #region Event Handlers (child controls)
    private void listView_ItemActivate(object sender, EventArgs e) {
      if (listView.SelectedItems.Count > 0) {
        IOperator op = listView.SelectedItems[0].Tag as IOperator;
        if (op != null) {
          MainFormManager.MainForm.ShowContent(op);
        }
      }
    }
    #endregion

    private bool suspended = false;
    public void SuspendUpdate() {
      suspended = true;
    }
    public void ResumeUpdate() {
      suspended = false;
      UpdateOperatorTrace();
    }
  }

}
